// Compact Disc Detect & Execute
// 
// Copyright(C) 2002-03 Eric Lathrop <eric@ericlathrop.com>
//

#ifndef __cdde_h__
#define __cdde_h__

#define NUM_DATA_TYPES 7
#define DATA_AUDIO 0
#define DATA_DATA 1
#define DATA_DVD 2
#define DATA_MIXED 3
#define DATA_BLANK 4
#define DATA_VCD 5
#define DATA_SVCD 6

#define DEFAULT_SLEEP_N 5000000
#define DEFAULT_SLEEP_S "5000000"


///////////////////////
// globals
///////////////////////

char * datatags[] = {"audio", "data", "dvd", "mixed", "blank", "vcd", "svcd"};

// for error checking
extern int errno;

// the config file filename
char * filename = NULL;

// keep checking drives
int keeprunning = 1;

// the number of children we have executed
int numchildren = 0;

// the polling delay
int delay;

#include "list.h"

typedef struct _drive
{
	char * filename;
	list * commands[NUM_DATA_TYPES];
	int dontexecute;
} drive;

list * drives = NULL;


///////////////////////////
// command line options and their defaults
///////////////////////////
int verbose = 0; // -v
char * configfile; // -c
int batchmode = 0; // -b
int dontrunfirst = 1; // -r


///////////////////////
// function prototypes
///////////////////////

// signal handler to find out when out child has exited
void sigchld(int signal);
// signal handler to find out when to re-read config file
void sighup(int signum);
// signal handler for cleanup and death
void sigint(int signum);

// fork and exec a command
// wait for it to exit before returning
void execute(const char * command);

// executes all commands for a related disc type
void executeall(drive * d, int cmdtype);

// determine if the device has a directory in its top level
//
// device is the filename of the special device file
// directory is the name of the directory to look for in the discs top directory
//
// returns nonzero if the file exists
// otherwise returns 0
int disc_has_dir(const char * device, const char * directory);

// see if a drive has a disc in it
// if so, determine type of disc
// and execute proper command
int checkdrive(drive * d);

// read a configuration file
int loadconfig(char * filename);
void parsedrive(xmlNodePtr cur);

// write a template configuation file
int saveconfig(char * filename);

// parse command line arguments
void parseargs(int argc, char ** argv);

// remove a drive from the list
void removedrive(unsigned int drivenum);

#endif
