// Compact Disc Detect & Execute
// 
// Copyright(C) 2002-03 Eric Lathrop <eric@ericlathrop.com>
//

#include <stdlib.h>
#include <string.h>

#include "string.h"

// find the first occurance of a character
// in a string (from start position)
//
int indexof(const char * str, char c, int start)
{
	int i;

	if (start < 0) start = 0;

	for (i=start; i<strlen(str); i++)
	{
		if (str[i] == c) return i;
	}
	return -1;
}

// create a smaller string
// from part of a larger string
// (note: user must free() the returned string when done)
//
char * substr(const char * str, unsigned int start, unsigned int length)
{
	char * result = "";
	int i;
	int stop = start+length;

	if ((start >= 0) && (start <= stop) && (stop <= strlen(str)))
	{
		result = malloc((stop-start+1)*sizeof(char));
		for (i=0; i<(stop-start); i++)
		{
			result[i] = str[start+i];
		}
		result[i] = '\0';
	}
	return result;
}

// creates a copy of a string but without
// leading or trailing whitespace;
// (note: user must free() the returned string when done)
//
char * trim(const char * str)
{
	int start = 0;
	int stop = strlen(str);

	while ((
			(str[start] == ' ') ||
			(str[start] == '\t') ||
			(str[start] == '\n')
		) && (start < strlen(str))) start++;
	while ((
			(str[stop-1] == ' ') ||
			(str[stop-1] == '\t') ||
			(str[stop-1] == '\n')
		) && (stop > -1)) stop--;

	return substr(str, start, stop-start);
}

// creates a copy of a string but without
// leading or trailing whitespace;
// (note: user must free() the returned string when done)
//
// this is different from trim() in that it destroys
// the original string, and replaces it with the trim()ed version
//
char * trim_destruct(char ** str)
{
	char * newstring = trim(*str);
	free(*str);
	*str = newstring;
	return newstring;
}

// replace all  occourances of oldstr with newstr in str
// (note: user must free() the returned string when done)
//
char * replace(const char * str, const char * oldstr, const char * newstr)
{
	char * pos;
	char * start;
	char * end;
	char * retval = substr(str, 0, strlen(str));
	char * temp;

	while ((pos = (char *)strstr(retval, oldstr)) != NULL)
	{
		start = substr(retval, 0, pos-retval);
		end = substr(pos, strlen(oldstr), strlen(pos)-strlen(oldstr));

		temp = malloc(sizeof(char) * (strlen(start) + strlen(newstr) + strlen(end) + 1));
		temp[0] = '\0';
		temp = (char *)strcat(temp, start);
		temp = (char *)strcat(temp, newstr);
		temp = (char *)strcat(temp, end);
		
		free(retval);
		retval = temp;
		free(start);
		free(end);
	}
	return retval;
}

// replace all  occourances of oldstr with newstr in str
// (note: user must free() the returned string when done)
//
// this is different from replace() in that it destroys
// the original string, and replaces it with the replace()ed version
//
char * replace_destruct(char ** str, const char * oldstr, const char * newstr)
{
	char * newstring = replace(*str, oldstr, newstr);
	free(*str);
	*str = newstring;
	return newstring;
}

// capitalizes all text in a string
// (note: user must free() the returned string when done)
//
char * toupper(const char * str)
{
	int i;
	int diff = 'A'-'a';
	char * newstr = (char *) malloc((strlen(str)+1) * sizeof(char));
	if (newstr == NULL)
	{
		printf("Error: couldn't allocate memory!\n");
		exit(1);
	}

	for (i=0; i<strlen(str); i++)
	{
		if ((str[i] >= 'a') && (str[i] <= 'z'))
		{
			newstr[i] = str[i]+diff;
		} else {
			newstr[i] = str[i];
		}
	}
	newstr[i] = '\0';
	return newstr;
}

// capitalizes all text in a string
// (note: user must free() the returned string when done)
//
// this is different from toupper() in that it destroys
// the original string, and replaces it with the toupper()ed version
//
char * toupper_destruct(char ** str)
{
	char * newstr = toupper(*str);
	free(*str);
	*str = newstr;
	return newstr;
}
